/* eslint-disable @stencil/own-methods-must-be-private, @stencil/own-props-must-be-private */
import { Component, Event, h, Listen, Method, Prop, State, Watch, } from '@stencil/core';
import { Logger } from '../../utils/Logger';
import { API_SERVER } from '../../utils/config';
import { classList, getTimezoneOffset, isHTMLAudioElement, validateEpisodeParams, } from '../../utils/utils';
import DeviceService from '../../utils/DeviceService';
import ClientService from '../../utils/ClientService';
import OptOutService from '../../utils/OptOutService';
import { ActionHandler } from './ActionHandler';
export class PdClient {
  constructor() {
    var _a;
    /**
     * If not using the audioElement prop, pass the current playback rate here
     */
    this.playbackRate = 1;
    /**
     * The User Agent your app uses.
     * Only necessary when integrating into an app with a custom user agent.
     * Will use the current browser's user agent if available.
     */
    this.userAgent = ((_a = window === null || window === void 0 ? void 0 : window.navigator) === null || _a === void 0 ? void 0 : _a.userAgent) || 'Poductivity-Client';
    /**
     * Optionally supply a <poductivity-opt-out-control> element positioned
     * anywhere in the DOM. Will show inside this element otherwise
     */
    this.optOutControl = null;
    /**
     * Use to disable the standard Poductivity subscribe behaviour if you wish
     * to capture and handle the subscribeToFeed event yourself
     */
    this.noSubscribe = false;
    /**
     * Optionally supply a <poductivity-messages> element positioned
     * anywhere in the DOM. Will show in this element otherwise
     */
    this.messagesElement = null;
    /**
     * Enable debug mode. Shows a status panel in the client and adds
     * extra console logging. For internal use only
     */
    this.debug = false;
    /**
     * Pass in a data object to use instead of contacting the API server
     * For internal/debug use only
     */
    this.data = null;
    this.episode = null;
    this.slots = [];
    this.optionGroups = [];
    this.optOut = '';
    this.loading = false;
    this.loaded = false;
    this.timestampI = 0;
    this.isPlayingI = false;
    this.internalDeviceIdentifier = '';
    this.dynamicOptionGroup = null;
    this.lastClose = 0;
    this.contiguousPlaybackStart = 0;
    this.playbackStarted = false;
    this.enablePrivacyControl = false;
    //rollup-plugin-version-injector will replace this
    this.version = '[VI]{version}[/VI]';
    this.clientService = new ClientService(this.userAgent);
    this.optOutService = OptOutService;
    this.logger = new Logger('PdClient', this.debug);
    this.actionHandler = new ActionHandler(this);
    this.lastTimestampUpdateAt = Date.now();
    this.audioElPlaying = this.audioElPlaying.bind(this);
    this.audioElPaused = this.audioElPaused.bind(this);
    this.audioElTimeUpdate = this.audioElTimeUpdate.bind(this);
    this.changeOptOut = this.changeOptOut.bind(this);
  }
  componentWillLoad() {
    this.updateAudioElementListeners(this.audioElement);
    this.updateExternalOptOutControlListeners(this.optOutControl);
    this.messagesElementChanged(this.messagesElement);
  }
  componentDidRender() {
    if (this.messagesElement) {
      this.messagesElement.optionGroups = this.currentOptionGroups();
    }
  }
  disconnectedCallback() {
    this.close();
  }
  /**
   * Set the current episode by feed URL and episode GUID.
   */
  async setEpisodeByFeed(feedUrl, episodeGuid) {
    return this.loadAll({
      feedUrl,
      episodeGuid,
    });
  }
  /**
   * Set the current episode by Poductivity Episode ID
   */
  async setEpisodeById(episodeId) {
    return this.loadAll({ id: episodeId });
  }
  /**
   * If integrating into an app, call this before destroying
   * the webview to allow the client to flush it's caches.
   * If integrating into the web, the client handles this itself
   * using the Browser Visibility API
   */
  async close() {
    const now = Date.now();
    const ignore = this.lastClose > now - 200;
    this.logger.log('Close method called', ignore);
    if (!ignore) {
      this.lastClose = now;
      await this.recordPlaybackPortion(this.contiguousPlaybackStart, this.timestampI);
      await this.clientService.flushResponseCache(this.internalDeviceIdentifier);
    }
  }
  /**
   * Is the client ready?
   */
  async isReady() {
    return this.hasLoadedData && !!this.internalDeviceIdentifier;
  }
  get hasLoadedData() {
    return !!this.data || this.loaded;
  }
  get subscribeData() {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    return {
      feedUrl: ((_b = (_a = this.episode) === null || _a === void 0 ? void 0 : _a.show) === null || _b === void 0 ? void 0 : _b.feedUrl) || '',
      feedProtocol: ((_d = (_c = this.episode) === null || _c === void 0 ? void 0 : _c.show) === null || _d === void 0 ? void 0 : _d.feedProtocol) || '',
      applePodcastsId: (_f = (_e = this.episode) === null || _e === void 0 ? void 0 : _e.show) === null || _f === void 0 ? void 0 : _f.applePodcastsId,
      spotifyId: (_h = (_g = this.episode) === null || _g === void 0 ? void 0 : _g.show) === null || _h === void 0 ? void 0 : _h.spotifyId,
    };
  }
  get playbackRateI() {
    var _a;
    return ((_a = this.audioElement) === null || _a === void 0 ? void 0 : _a.playbackRate) || this.playbackRate;
  }
  get wrapperClasses() {
    return classList({
      'poductivity-wrapper': true,
      active: this.currentActionableOptionGroups().length ||
        (!this.messagesElement && this.currentOptionGroups().length),
    });
  }
  async staticDataChanged(data) {
    await this.setDeviceIdentifier(this.deviceIdentifier);
    this.slots = data.slots;
    this.optionGroups = data.optionGroups;
    this.episode = data.episode;
  }
  async loadAll(params) {
    validateEpisodeParams(params);
    try {
      this.loading = true;
      await this.setDeviceIdentifier(this.deviceIdentifier);
      await this.loadData(params);
      if (this.episode && this.episode.id) {
        this.subscribeToOptOuts();
        this.loaded = true;
      }
    }
    catch (e) {
      this.logger.error('Unable to load episode', e);
      this.loadingEpisodeDataFailed.emit(e);
    }
    this.loading = false;
  }
  async loadData(params) {
    if (!this.data) {
      this.slots = [];
      this.optionGroups = [];
      this.dynamicOptionGroup = null;
      this.loadingEpisodeData.emit();
      const data = await this.clientService.getEpisodeData(params, this.internalDeviceIdentifier);
      this.slots = data.slots || [];
      this.optionGroups = data.optionGroups || [];
      this.episode = data.episode || null;
      this.loadedEpisodeData.emit(data);
    }
  }
  subscribeToOptOuts() {
    var _a;
    if (this.enablePrivacyControl) {
      (_a = this.privacySubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
      this.privacySubscription = this.optOutService
        .getOptOut(this.internalDeviceIdentifier, this.episode.showId, this.userAgent)
        .subscribe((optOut) => (this.optOut = optOut));
    }
  }
  optOutChanged(val) {
    if (this.optOutControl) {
      this.optOutControl.optOut = val;
    }
  }
  onPageHide() {
    this.close();
  }
  onVisibilityChange() {
    if (document.visibilityState === 'hidden') {
      this.close();
    }
  }
  async changeOptOut(event) {
    var _a;
    const { detail } = event;
    this.logger.log('Opt out changed', detail);
    await this.optOutService.saveOptOut(detail.optOut, (_a = this === null || this === void 0 ? void 0 : this.episode) === null || _a === void 0 ? void 0 : _a.showId, this.internalDeviceIdentifier, this.userAgent);
  }
  debugChanged(debug) {
    this.logger.setDebug(!!debug);
  }
  userAgentChanged(userAgent) {
    this.clientService.setUserAgent(userAgent);
  }
  async setDeviceIdentifier(suppliedDeviceIdentifier) {
    if (suppliedDeviceIdentifier) {
      this.internalDeviceIdentifier = suppliedDeviceIdentifier;
    }
    else {
      const { deviceIdentifier } = await DeviceService.getDeviceInfo();
      this.internalDeviceIdentifier = deviceIdentifier;
    }
  }
  updateAudioElementListeners(newEl, oldEl) {
    if (isHTMLAudioElement(oldEl)) {
      this.logger.log('Detaching audio element listeners', oldEl);
      oldEl.removeEventListener('timeupdate', this.audioElTimeUpdate);
      oldEl.removeEventListener('play', this.audioElPlaying);
      oldEl.removeEventListener('pause', this.audioElPaused);
    }
    if (isHTMLAudioElement(newEl)) {
      this.logger.log('Attaching audio element listeners', newEl);
      newEl.addEventListener('timeupdate', this.audioElTimeUpdate);
      newEl.addEventListener('play', this.audioElPlaying);
      newEl.addEventListener('pause', this.audioElPaused);
    }
  }
  updateExternalOptOutControlListeners(newEl, oldEl) {
    if (oldEl) {
      this.logger.log('Detaching opt out control listeners');
      oldEl.removeEventListener('optOutChanged', this.changeOptOut);
    }
    if (newEl) {
      this.logger.log('Attaching opt out control listeners');
      newEl.addEventListener('optOutChanged', this.changeOptOut);
      newEl.optOut = this.optOut;
    }
  }
  audioElPlaying() {
    this.isPlayingI = true;
  }
  audioElPaused() {
    this.isPlayingI = false;
  }
  isPlayingPropChanged() {
    if (typeof this.isPlaying !== 'boolean') {
      this.logger.warn('Poductivity: Invalid isPlaying prop');
    }
    else {
      this.isPlayingI = this.isPlaying;
    }
  }
  internalIsPlayingChanged(isPlaying) {
    this.logger.log('Playing state changed', isPlaying);
    if (!this.playbackStarted) {
      this.contiguousPlaybackStart = this.timestampI;
      this.playbackStarted = true;
    }
    if (!isPlaying) {
      this.recordPlaybackPortion(this.contiguousPlaybackStart, this.timestampI);
    }
  }
  audioElTimeUpdate(event) {
    this.timestampI = event.target.currentTime || 0;
  }
  timestampPropChanged() {
    if (typeof this.timestamp !== 'number') {
      this.logger.warn('Poductivity: Invalid timestamp prop');
    }
    else {
      this.timestampI = this.timestamp;
    }
  }
  internalTimestampChanged(newTimestamp, oldTimestamp) {
    const timeStampDiff = (newTimestamp - oldTimestamp) * 1000;
    const timeDiff = Date.now() - this.lastTimestampUpdateAt;
    //apply fuzz factor of 1% + 50ms
    const seeked = timeStampDiff < 0 ||
      timeStampDiff > timeDiff * this.playbackRateI * 1.01 + 50;
    if (seeked) {
      this.logger.log('Seek detected', oldTimestamp, newTimestamp, (timeStampDiff - timeDiff) / 1000);
      this.recordPlaybackPortion(this.contiguousPlaybackStart, oldTimestamp);
      this.contiguousPlaybackStart = newTimestamp;
    }
    this.lastTimestampUpdateAt = Date.now();
  }
  messagesElementChanged(messagesElement) {
    if (messagesElement) {
      this.logger.log('messages element changed, setting option groups', messagesElement);
      messagesElement.optionGroups = this.currentOptionGroups();
    }
  }
  currentSlots(timestamp) {
    const ts = typeof timestamp === 'number' ? timestamp : this.timestampI;
    return this.slots.filter((s) => s.start <= ts && s.end > ts);
  }
  currentOptionGroups(timestamp) {
    const ts = typeof timestamp === 'number' ? timestamp : this.timestampI;
    const slotIds = this.currentSlots(ts).map((s) => s.id);
    const ogs = this.optionGroups.filter((og) => slotIds.indexOf(og.slotId) !== -1);
    if (this.dynamicOptionGroup) {
      ogs.unshift(this.dynamicOptionGroup);
    }
    return ogs;
  }
  currentActionableOptionGroups(timestamp) {
    const ts = typeof timestamp === 'number' ? timestamp : this.timestampI;
    return this.currentOptionGroups(ts).filter((og) => !og.hide && !(og.oneTime && og.previouslyActed));
  }
  editOptionGroup(optionGroupId, changes) {
    const optionGroups = [...this.optionGroups];
    const index = optionGroups.findIndex((og) => og.id === optionGroupId);
    if (index === -1) {
      return null;
    }
    optionGroups[index] = Object.assign(Object.assign({}, optionGroups[index]), changes);
    this.optionGroups = optionGroups;
    return optionGroups[index];
  }
  editOption(optionGroupId, optionId, changes) {
    const optionGroups = [...this.optionGroups];
    const optionGroupIndex = optionGroups.findIndex((og) => og.id === optionGroupId);
    if (optionGroupIndex === -1) {
      return null;
    }
    const options = optionGroups[optionGroupIndex].options;
    const optionIndex = options.findIndex((o) => o.id === optionId);
    if (optionIndex === -1) {
      return null;
    }
    options[optionIndex] = Object.assign(Object.assign({}, options[optionIndex]), changes);
    optionGroups[optionGroupIndex] = Object.assign(Object.assign({}, optionGroups[optionGroupIndex]), { options: options });
    this.optionGroups = optionGroups;
    return options[optionIndex];
  }
  async recordPlaybackPortion(start, end) {
    // Ignore delta < 1 second
    if (end - start < 1) {
      return;
    }
    const coveredSlots = this.slots.filter((s) => s.start >= start - 0.5 && s.end <= end + 0.5);
    const plays = coveredSlots.map((s) => ({
      slotId: s.id,
      occurredAt: Math.round(Date.now() - (end - s.end) * 100),
      localTimezoneOffset: getTimezoneOffset(),
      playbackRate: this.playbackRateI,
    }));
    this.logger.log('Portion played', coveredSlots, start, end);
    if (plays.length) {
      await this.clientService.recordPlays(plays);
    }
  }
  async recordAction(option, actionIndex) {
    var _a, _b, _c;
    if (!this.episode) {
      throw 'Episode not loaded';
    }
    // actionIndex === -1 refers to the first action being started,
    // so all step counts have to be shifted by 1
    const response = {
      optionId: option === null || option === void 0 ? void 0 : option.id,
      actionId: (_a = option === null || option === void 0 ? void 0 : option.clientActions[actionIndex !== -1 ? actionIndex : 0]) === null || _a === void 0 ? void 0 : _a.actionId,
      timestamp: ((_b = this.audioElement) === null || _b === void 0 ? void 0 : _b.currentTime) || 0,
      step: actionIndex + 2,
      totalSteps: ((_c = option === null || option === void 0 ? void 0 : option.clientActions) === null || _c === void 0 ? void 0 : _c.length) + 1,
      occurredAt: new Date().getTime(),
      localTimezoneOffset: getTimezoneOffset(),
    };
    await this.clientService.recordResponse(response);
  }
  async perform(option, optionGroup) {
    this.logger.log('Option initiated', option);
    this.performActionEvent.emit({
      option,
      optionGroup,
    });
    try {
      await this.recordAction(option, -1);
      for (let index = 0; index < option.clientActions.length; index++) {
        const action = option.clientActions[index];
        if (!(action.type in this.actionHandler)) {
          throw 'Action type not supported by this client version';
        }
        this.logger.log('Starting action', action);
        await this.actionHandler[action.type](action, option, optionGroup);
        await this.recordAction(option, index);
        this.logger.log('Action completed', action);
      }
      this.editOptionGroup(optionGroup.id, {
        previouslyActed: true,
      });
      this.logger.log('All actions performed for option', option);
    }
    catch (e) {
      this.logger.log('Action not completed', e);
    }
    this.clientService.flushResponseCache(this.internalDeviceIdentifier);
  }
  controlAudio(action, val) {
    this.audioControlEvent.emit({
      action,
      val,
    });
    if (isHTMLAudioElement(this.audioElement)) {
      switch (action) {
        case 'seekAbsolute':
          this.audioElement.currentTime = val || 0;
          break;
        case 'seekRelative':
          this.audioElement.currentTime += val || 0;
          break;
        case 'pause':
          this.audioElement.pause();
          break;
        case 'resume':
          this.audioElement.play();
          break;
      }
    }
  }
  getDebugPanel() {
    var _a;
    return (this.debug && (h("pre", { class: "pd-debug" },
      h("div", null,
        "SDK Version: ",
        this.version),
      h("div", null,
        "Device Identifier: ",
        this.internalDeviceIdentifier),
      h("div", null,
        "Client ready: ",
        this.isReady() ? 'true' : 'false'),
      h("div", null,
        "AudioElement ready:",
        ((_a = this.audioElement) === null || _a === void 0 ? void 0 : _a.isConnected) ? 'true' : 'false'),
      this.enablePrivacyControl && (h("div", null,
        "Opt out: ",
        this.optOut || 'false')),
      h("div", null,
        "API Server: ",
        API_SERVER),
      h("div", null,
        "Current slots: ",
        this.currentSlots().length),
      h("div", null,
        "Current option groups: ",
        this.currentOptionGroups().length),
      h("div", null,
        "Current time: ",
        this.timestampI))));
  }
  getCurrentOptionGroupsUI() {
    if (this.currentOptionGroups().length === 0) {
      return (h("div", { class: "pd-option-group-null", onClick: () => this.recordAction() }, "\u00A0"));
    }
    else {
      return this.currentOptionGroups().map((optionGroup) => {
        const classes = classList({
          'pd-option-group': true,
          'pd-option-group-multiple': optionGroup.options.length > 1,
        });
        return (h("div", { key: optionGroup.id, class: classes },
          !this.messagesElement && (h("poductivity-messages", { optionGroups: optionGroup })),
          this.getOptionGroupButtons(optionGroup)));
      });
    }
  }
  getOptionGroupButtons(optionGroup) {
    return (!optionGroup.hide &&
      !(optionGroup.oneTime && optionGroup.previouslyActed) && (h("div", { class: "pd-option-group-options" }, (optionGroup.options || []).map((option) => {
      return (h("button", { key: option.id, class: "pd-button pd-button-large pd-option", onClick: () => this.perform(option, optionGroup) }, option.title));
    }))));
  }
  render() {
    var _a;
    return (h("div", { class: this.wrapperClasses },
      this.getDebugPanel(),
      this.isReady() && this.getCurrentOptionGroupsUI(),
      h("poductivity-modal", { ref: (el) => (this.modalElement = el), debug: this.debug }),
      !this.noSubscribe && ((_a = this.episode) === null || _a === void 0 ? void 0 : _a.show) && (h("poductivity-subscribe", { ref: (el) => (this.subscribeElement = el), showData: this.subscribeData, userAgent: this.userAgent, debug: this.debug })),
      !this.optOutControl && this.enablePrivacyControl && (h("poductivity-opt-out-control", { optOut: this.optOut, up: true, right: true }))));
  }
  static get is() { return "poductivity-client"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-client.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-client.css"]
  }; }
  static get properties() { return {
    "audioElement": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLAudioElement",
        "resolved": "HTMLAudioElement",
        "references": {
          "HTMLAudioElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Pass a reference to a <audio> element to have the\nclient attach the necessary listeners and perform\ncontrol action for you"
      }
    },
    "timestamp": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If not using the audioElement prop, pass the current audio timestamp here"
      },
      "attribute": "timestamp",
      "reflect": false
    },
    "isPlaying": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If not using the audioElement prop, pass the current playback state here"
      },
      "attribute": "is-playing",
      "reflect": false
    },
    "playbackRate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If not using the audioElement prop, pass the current playback rate here"
      },
      "attribute": "playback-rate",
      "reflect": false,
      "defaultValue": "1"
    },
    "deviceIdentifier": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If integrating into an app, pass the device's native advertising ID here.\nIf integrating into a web app, the client will handle generation\nand persistence of a device identifier for you"
      },
      "attribute": "device-identifier",
      "reflect": false
    },
    "userAgent": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The User Agent your app uses.\nOnly necessary when integrating into an app with a custom user agent.\nWill use the current browser's user agent if available."
      },
      "attribute": "user-agent",
      "reflect": false,
      "defaultValue": "window?.navigator?.userAgent || 'Poductivity-Client'"
    },
    "optOutControl": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLPoductivityOptOutControlElement | null",
        "resolved": "HTMLPoductivityOptOutControlElement",
        "references": {
          "HTMLPoductivityOptOutControlElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Optionally supply a <poductivity-opt-out-control> element positioned\nanywhere in the DOM. Will show inside this element otherwise"
      },
      "defaultValue": "null"
    },
    "noSubscribe": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Use to disable the standard Poductivity subscribe behaviour if you wish\nto capture and handle the subscribeToFeed event yourself"
      },
      "attribute": "no-subscribe",
      "reflect": false,
      "defaultValue": "false"
    },
    "messagesElement": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLPoductivityMessagesElement | null",
        "resolved": "HTMLPoductivityMessagesElement",
        "references": {
          "HTMLPoductivityMessagesElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Optionally supply a <poductivity-messages> element positioned\nanywhere in the DOM. Will show in this element otherwise"
      },
      "defaultValue": "null"
    },
    "debug": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable debug mode. Shows a status panel in the client and adds\nextra console logging. For internal use only"
      },
      "attribute": "debug",
      "reflect": false,
      "defaultValue": "false"
    },
    "data": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "PdData | null",
        "resolved": "PdData",
        "references": {
          "PdData": {
            "location": "import",
            "path": "../../slot.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Pass in a data object to use instead of contacting the API server\nFor internal/debug use only"
      },
      "defaultValue": "null"
    }
  }; }
  static get states() { return {
    "episode": {},
    "slots": {},
    "optionGroups": {},
    "optOut": {},
    "loading": {},
    "loaded": {},
    "timestampI": {},
    "isPlayingI": {},
    "internalDeviceIdentifier": {},
    "dynamicOptionGroup": {}
  }; }
  static get events() { return [{
      "method": "loadingEpisodeData",
      "name": "loadingEpisodeData",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the episode data is about to load"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "loadedEpisodeData",
      "name": "loadedEpisodeData",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the episode data is loaded"
      },
      "complexType": {
        "original": "LoadedEpisodeData",
        "resolved": "LoadedEpisodeData",
        "references": {
          "LoadedEpisodeData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }, {
      "method": "loadingEpisodeDataFailed",
      "name": "loadingEpisodeDataFailed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted if the episode data fails to load"
      },
      "complexType": {
        "original": "LoadingEpisodeDataFailed",
        "resolved": "LoadingEpisodeDataFailed",
        "references": {
          "LoadingEpisodeDataFailed": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }, {
      "method": "performActionEvent",
      "name": "performAction",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when an action is performed\nOnly required if you want your app to also perform an action as a result"
      },
      "complexType": {
        "original": "PerformActionData",
        "resolved": "PerformActionData",
        "references": {
          "PerformActionData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }, {
      "method": "audioControlEvent",
      "name": "audioControl",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when an action requests a change to audio control.\nOnly required if not using the audioElement prop"
      },
      "complexType": {
        "original": "AudioControlEventData",
        "resolved": "AudioControlEventData",
        "references": {
          "AudioControlEventData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }, {
      "method": "openExternalUrl",
      "name": "openExternalUrl",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when an action requests to open a URL in a new window.\nOnly required when integrating into an app that won't support window.open"
      },
      "complexType": {
        "original": "OpenExternalUrlData",
        "resolved": "OpenExternalUrlData",
        "references": {
          "OpenExternalUrlData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }, {
      "method": "subscribeToFeed",
      "name": "subscribeToFeed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when an action requests to subscribe to a feed.\nOnly required when wishing to override the default modal"
      },
      "complexType": {
        "original": "FeedSubscribeData",
        "resolved": "FeedSubscribeData",
        "references": {
          "FeedSubscribeData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }]; }
  static get methods() { return {
    "setEpisodeByFeed": {
      "complexType": {
        "signature": "(feedUrl: string, episodeGuid: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set the current episode by feed URL and episode GUID.",
        "tags": []
      }
    },
    "setEpisodeById": {
      "complexType": {
        "signature": "(episodeId: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set the current episode by Poductivity Episode ID",
        "tags": []
      }
    },
    "close": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "If integrating into an app, call this before destroying\nthe webview to allow the client to flush it's caches.\nIf integrating into the web, the client handles this itself\nusing the Browser Visibility API",
        "tags": []
      }
    },
    "isReady": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Is the client ready?",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "data",
      "methodName": "staticDataChanged"
    }, {
      "propName": "optOut",
      "methodName": "optOutChanged"
    }, {
      "propName": "debug",
      "methodName": "debugChanged"
    }, {
      "propName": "userAgent",
      "methodName": "userAgentChanged"
    }, {
      "propName": "deviceIdentifier",
      "methodName": "setDeviceIdentifier"
    }, {
      "propName": "audioElement",
      "methodName": "updateAudioElementListeners"
    }, {
      "propName": "optOutControl",
      "methodName": "updateExternalOptOutControlListeners"
    }, {
      "propName": "isPlaying",
      "methodName": "isPlayingPropChanged"
    }, {
      "propName": "isPlayingI",
      "methodName": "internalIsPlayingChanged"
    }, {
      "propName": "timestamp",
      "methodName": "timestampPropChanged"
    }, {
      "propName": "timestampI",
      "methodName": "internalTimestampChanged"
    }, {
      "propName": "messagesElement",
      "methodName": "messagesElementChanged"
    }]; }
  static get listeners() { return [{
      "name": "pagehide",
      "method": "onPageHide",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "visibilitychange",
      "method": "onVisibilityChange",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "optOutChanged",
      "method": "changeOptOut",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
