import { Component, h, Host, Method, Prop, readTask, State, } from '@stencil/core';
import ResizeObserver from 'resize-observer-polyfill';
import { classList } from '../../utils/utils';
/**
 * @slot default - The content to slide
 */
export class PdMarquee {
  constructor() {
    /**
     * How many px should move per second?
     */
    this.speed = 40;
    /**
     * Enable the animation
     */
    this.active = false;
    /**
     * A unique ID for the component. Will be auto generated if not supplied
     */
    this.uid = '';
    this.overflow = 0;
    this.fallbackId = String(Math.floor(Math.random() * 1000000));
  }
  componentDidLoad() {
    this.resizeObserver = new ResizeObserver(() => {
      this.setOverflow();
    });
    this.resizeObserver.observe(this.contentEl);
  }
  disconnectedCallback() {
    this.resizeObserver.disconnect();
  }
  componentWillUpdate() {
    this.setOverflow();
  }
  /**
   * Force a recalculation of the overflow
   */
  async setOverflow() {
    readTask(() => {
      this.overflow =
        (this.contentEl.scrollWidth || 0) - (this.contentEl.offsetWidth || 0);
    });
  }
  get contentClasses() {
    return classList({
      'pd-marquee-content': true,
      active: this.active,
    });
  }
  get contentStyle() {
    return this.overflow
      ? {
        animationName: this.animationName,
        animationDelay: '500ms',
        animationDirection: 'alternate',
        animationDuration: `${this.duration}ms`,
        animationIterationCount: 'infinite',
        animationTimingFunction: 'ease-in-out',
        animationPlayState: this.active ? 'running' : 'paused',
      }
      : {};
  }
  get internalId() {
    return this.uid || this.fallbackId;
  }
  get animationName() {
    return `pd-marquee-${this.internalId}`;
  }
  get duration() {
    const dur = (this.overflow / this.speed) * 1000;
    return dur > 1000 ? dur : 1000;
  }
  render() {
    return (h(Host, null,
      h("style", null, this.overflow
        ? `
          @keyframes ${this.animationName} {
            from { transform: translateX(0) }
            to { transform: translateX(-${this.overflow}px) }
          }
        `
        : ''),
      h("div", { ref: (el) => (this.contentEl = el), class: this.contentClasses, style: this.contentStyle },
        h("slot", null))));
  }
  static get is() { return "poductivity-marquee"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-marquee.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-marquee.css"]
  }; }
  static get properties() { return {
    "speed": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "How many px should move per second?"
      },
      "attribute": "speed",
      "reflect": false,
      "defaultValue": "40"
    },
    "active": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable the animation"
      },
      "attribute": "active",
      "reflect": false,
      "defaultValue": "false"
    },
    "uid": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "A unique ID for the component. Will be auto generated if not supplied"
      },
      "attribute": "uid",
      "reflect": false,
      "defaultValue": "''"
    }
  }; }
  static get states() { return {
    "overflow": {}
  }; }
  static get methods() { return {
    "setOverflow": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Force a recalculation of the overflow",
        "tags": []
      }
    }
  }; }
}
