import { Component, h, Host, Listen, Method, Prop, State } from '@stencil/core';
import iframeResize from 'iframe-resizer/js/iframeResizer';
import iconX from 'boxicons/svg/regular/bx-x.svg';
import iconLoading from 'boxicons/svg/regular/bx-loader-alt.svg';
import { Logger } from '../../utils/Logger';
import { addQueryParam, classList, uuid } from '../../utils/utils';
import { checkMessageOrigin } from '../../utils/httpUtils';
import { API_SERVER, LISTENERS_SERVER } from '../../utils/config';
/**
 * @slot default - Optional HTML content for the modal if not using the iframe
 */
export class PdModal {
  constructor() {
    /**
     * Position the modal absolute rather than fixed
     */
    this.absolute = false;
    /**
     * Fill all available space - no visible backdrop
     */
    this.fill = false;
    /**
     * Debug mode
     */
    this.debug = false;
    this.isShown = false;
    this.isLoadingFrame = false;
    this.frameSrc = null;
    this.resolveI = null;
    this.rejectI = null;
    this.iframeResizeInitiated = false;
    this.instanceId = uuid();
    this.logger = new Logger('PdModal', this.debug);
    this.backdropClicked = this.backdropClicked.bind(this);
    this.forceClose = this.forceClose.bind(this);
    this.frameLoaded = this.frameLoaded.bind(this);
  }
  componentDidRender() {
    if (this.frameElement && !this.iframeResizeInitiated) {
      iframeResize({
        checkOrigin: [LISTENERS_SERVER],
        log: this.debug,
        scrolling: true,
        minHeight: 150,
      }, this.frameElement);
      this.iframeResizeInitiated = true;
    }
  }
  /**
   * Show the modal
   * @param frameSrc Optional source for the iframe
   */
  async show(frameSrc = null) {
    // Dismiss any currently active modal
    this.hide(false);
    this.isLoadingFrame = !!frameSrc;
    this.isShown = true;
    this.frameSrc = frameSrc;
    return new Promise((resolve, reject) => {
      this.resolveI = resolve;
      this.rejectI = reject;
    });
  }
  /**
   * Hide the modal. Specify whether the action was successful
   * @param message
   */
  async hide(message) {
    if (this.resolveI && this.rejectI) {
      message ? this.resolveI(message) : this.rejectI(message);
    }
    this.isShown = false;
    this.iframeResizeInitiated = false;
  }
  /**
   * Hide the modal with a failed action result
   */
  async forceClose() {
    this.hide(false);
  }
  receiveMessage(e) {
    var _a;
    if (checkMessageOrigin(e.origin) &&
      ((_a = e.data) === null || _a === void 0 ? void 0 : _a.instanceId) === this.instanceId) {
      this.logger.log('Message received from frame', e.data);
      this.hide(e.data.success || false);
    }
  }
  get frameAllowPolicy() {
    return `payment ${API_SERVER};`;
  }
  get fullFrameSrc() {
    return addQueryParam(this.frameSrc, 'instanceId', this.instanceId);
  }
  get wrapperClasses() {
    return classList({
      'pd-modal-show': this.isShown,
      'pd-modal-absolute': this.absolute,
      'pd-modal-fill': this.fill,
    });
  }
  backdropClicked(event) {
    if (event.target.tagName.toLowerCase() ===
      'poductivity-modal') {
      this.hide(false);
    }
  }
  frameLoaded() {
    this.isLoadingFrame = false;
  }
  render() {
    return (h(Host, { class: this.wrapperClasses, onClick: this.backdropClicked },
      h("div", { class: "pd-modal" },
        h("button", { title: "Close", class: "pd-button pd-modal-close-button", onClick: this.forceClose, innerHTML: iconX }),
        this.frameSrc && this.isLoadingFrame && (h("div", { class: "pd-modal-loading", innerHTML: iconLoading })),
        this.frameSrc && this.isShown && (h("iframe", { ref: (el) => (this.frameElement = el), src: this.fullFrameSrc, class: "pd-modal-frame", onLoad: this.frameLoaded, allow: this.frameAllowPolicy })),
        h("div", { class: "pd-modal-slot" },
          h("slot", null)))));
  }
  static get is() { return "poductivity-modal"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-modal.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-modal.css"]
  }; }
  static get properties() { return {
    "absolute": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Position the modal absolute rather than fixed"
      },
      "attribute": "absolute",
      "reflect": false,
      "defaultValue": "false"
    },
    "fill": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Fill all available space - no visible backdrop"
      },
      "attribute": "fill",
      "reflect": false,
      "defaultValue": "false"
    },
    "debug": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Debug mode"
      },
      "attribute": "debug",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "isShown": {},
    "isLoadingFrame": {},
    "frameSrc": {},
    "resolveI": {},
    "rejectI": {}
  }; }
  static get methods() { return {
    "show": {
      "complexType": {
        "signature": "(frameSrc?: string | null) => Promise<boolean>",
        "parameters": [{
            "tags": [{
                "text": "frameSrc Optional source for the iframe",
                "name": "param"
              }],
            "text": "Optional source for the iframe"
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Show the modal",
        "tags": [{
            "name": "param",
            "text": "frameSrc Optional source for the iframe"
          }]
      }
    },
    "hide": {
      "complexType": {
        "signature": "(message: boolean) => Promise<void>",
        "parameters": [{
            "tags": [{
                "text": "message",
                "name": "param"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Hide the modal. Specify whether the action was successful",
        "tags": [{
            "name": "param",
            "text": "message"
          }]
      }
    },
    "forceClose": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Hide the modal with a failed action result",
        "tags": []
      }
    }
  }; }
  static get listeners() { return [{
      "name": "message",
      "method": "receiveMessage",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}
