import { Component, Event, getAssetPath, h, Method, Prop, State, Watch, } from '@stencil/core';
import { classList, guessImageMimeFromPath, secsToHr } from '../../utils/utils';
import logoSvg from './assets/pd-icon.svg';
import iconPause from 'boxicons/svg/regular/bx-pause.svg';
import iconPlay from 'boxicons/svg/regular/bx-play.svg';
import iconSeekBack from './assets/plyr-control-bck15.svg';
import iconSeekForward from './assets/plyr-control-fwd15.svg';
import { Logger } from '../../utils/Logger';
import FontLoader from '../../utils/FontLoader';
import { karlaUnicodeRange } from './assets/karla';
export class PdPlayer {
  constructor() {
    /**
     * Should the player be horizontal?
     */
    this.horizontal = false;
    /**
     * Specify the theme to use. Defaults to 'light'.
     */
    this.theme = 'light';
    /**
     * Use the Operating System's default UI font instead of
     * the Poductivity brand font
     */
    this.useOsFont = false;
    /**
     * Enable Debug mode. For internal use only
     */
    this.debug = false;
    this.episode = null;
    this.isPlaying = false;
    this.isBuffering = false;
    this.currentTime = 0;
    this.playbackRate = 1;
    this.isLoading = true;
    this.loadingFailed = false;
    this.playError = false;
    this.isSeeking = false;
    this.audio = null;
    this.client = null;
    this.notesModal = null;
    this.messagesElement = null;
    this.optOutControl = null;
    this.enablePrivacyControl = false;
    this.logger = new Logger('PdPlayer', this.debug);
    this.onDataLoading = this.onDataLoading.bind(this);
    this.onDataLoaded = this.onDataLoaded.bind(this);
    this.onDataLoadingFailed = this.onDataLoadingFailed.bind(this);
    this.onTimeUpdate = this.onTimeUpdate.bind(this);
    this.onPlaybackRateChanged = this.onPlaybackRateChanged.bind(this);
    this.onSeeked = this.onSeeked.bind(this);
    this.onSeeking = this.onSeeking.bind(this);
    this.togglePlay = this.togglePlay.bind(this);
    this.seekBack = this.seekBack.bind(this);
    this.seekForward = this.seekForward.bind(this);
    this.onPlay = this.onPlay.bind(this);
    this.onPlaying = this.onPlaying.bind(this);
    this.onBuffering = this.onBuffering.bind(this);
    this.onPause = this.onPause.bind(this);
    this.showEpisodeNotes = this.showEpisodeNotes.bind(this);
    this.hideEpisodeNotes = this.hideEpisodeNotes.bind(this);
  }
  componentWillLoad() {
    if (!this.useOsFont) {
      FontLoader.load({
        family: 'Karla',
        style: 'normal',
        weight: '200 800',
        unicodeRange: karlaUnicodeRange,
        woff2Url: getAssetPath('./assets/karla-latin-variable-wghtOnly-normal.woff2'),
      });
    }
  }
  componentDidLoad() {
    this.setEpisodeParams();
  }
  componentDidRender() {
    this.client.audioElement = this.audio;
    this.client.debug = this.debug;
    if (this.optOutControl) {
      this.client.optOutControl = this.optOutControl;
    }
    this.client.messagesElement = this.messagesElement;
  }
  get duration() {
    var _a;
    if ((_a = this.audio) === null || _a === void 0 ? void 0 : _a.duration) {
      return this.audio.duration;
    }
    if (this.episode) {
      return this.episode.duration;
    }
    return 0;
  }
  get durationFormatted() {
    return secsToHr(this.duration);
  }
  get currentTimeFormatted() {
    return secsToHr(this.currentTime);
  }
  get subtitle() {
    var _a;
    if (this.isLoading || this.loadingFailed) {
      return String.fromCharCode(160);
    }
    if (this.playError) {
      return 'Unable to play episode';
    }
    return ((_a = this.episode) === null || _a === void 0 ? void 0 : _a.title) || 'Untitled Episode';
  }
  get title() {
    var _a, _b;
    if (this.loadingFailed) {
      return 'Unable to load episode';
    }
    if (this.isLoading) {
      return 'Loading episode' + String.fromCharCode(8230);
    }
    return ((_b = (_a = this.episode) === null || _a === void 0 ? void 0 : _a.show) === null || _b === void 0 ? void 0 : _b.title) || 'Untitled Show';
  }
  get ready() {
    return !!this.episode && !this.isLoading && !this.loadingFailed;
  }
  get wrapperClasses() {
    const classes = {
      'poductivity-player-wrapper': true,
      loading: this.isLoading,
      buffering: this.isBuffering,
      horizontal: this.horizontal,
    };
    if (this.theme) {
      classes[`theme-${this.theme}`] = true;
    }
    return classList(classes);
  }
  /**
   * Begin or resume playback
   */
  async play() {
    var _a;
    this.registerMediaSession();
    try {
      // NETWORK_NO_SOURCE === 3
      if (this.episode && ((_a = this.audio) === null || _a === void 0 ? void 0 : _a.networkState) === 3) {
        this.logger.log('Initialising audio element');
        this.audio.load();
      }
      await this.audio.play();
      this.playError = false;
    }
    catch (e) {
      if ('call to pause'.indexOf(e.message) === -1) {
        this.logger.log('Unable to play', e);
        this.playError = true;
        this.isBuffering = false;
      }
    }
  }
  /**
   * Pause playback
   */
  async pause() {
    return this.audio.pause();
  }
  /**
   * Either play or pause playback, depending on the current playback state
   */
  async togglePlay() {
    return this.isPlaying ? this.pause() : this.play();
  }
  /**
   * Seek to a specific timestamp in seconds
   */
  async seekTo(timestamp, fastSeek = false) {
    const sanitized = (typeof timestamp === 'number' ? timestamp : parseFloat(timestamp)) || 0;
    // Explicitly set current time in case audio el isn't yet initialised
    this.currentTime = sanitized;
    if (fastSeek && 'fastSeek' in this.audio) {
      this.audio.fastSeek(sanitized);
    }
    else {
      this.audio.currentTime = sanitized;
    }
  }
  /**
   * Seek to a timestamp relative to the current playback position in seconds
   */
  async seekRelative(seconds) {
    const parsed = (typeof seconds === 'number' ? seconds : parseFloat(seconds)) || 0;
    this.seekTo(this.currentTime + parsed);
  }
  /**
   * Show the episode notes panel
   */
  async setPlaybackRate(playbackRate) {
    if (typeof playbackRate === 'number') {
      this.playbackRate = playbackRate;
      this.audio.playbackRate = this.playbackRate;
    }
  }
  /**
   * Show the episode notes panel
   */
  async showEpisodeNotes() {
    this.notesModal.show().catch(() => { });
  }
  /**
   * Hide the episode notes panel
   */
  async hideEpisodeNotes() {
    this.notesModal.hide(true);
  }
  async setEpisodeParams() {
    if (this.episodeId) {
      await this.client.setEpisodeById(this.episodeId);
    }
    else if (this.episodeGuid && this.feedUrl) {
      await this.client.setEpisodeByFeed(this.feedUrl, this.episodeGuid);
    }
  }
  onDataLoading() {
    this.isLoading = true;
    this.loadingFailed = false;
  }
  onDataLoaded(event) {
    this.episode = event.detail.episode;
    this.isLoading = false;
    this.loadingFailed = !event.detail.episode;
  }
  onDataLoadingFailed() {
    this.loadingFailed = true;
    this.isLoading = false;
  }
  async seekForward() {
    await this.seekRelative(15);
  }
  async seekBack() {
    await this.seekRelative(-15);
  }
  onTimeUpdate(e) {
    if (!this.isSeeking) {
      this.currentTime = e.target.currentTime || 0;
    }
    this.updateMediaSessionState();
  }
  onPlaybackRateChanged(e) {
    this.setPlaybackRate(e.detail.playbackRate);
    this.updateMediaSessionState();
  }
  onSeeking(e) {
    this.isSeeking = e.detail.isSeeking;
  }
  onSeeked(e) {
    this.seekTo(e.detail.value, true);
  }
  onPlay() {
    this.isPlaying = true;
    this.pdPlayback.emit({
      playing: this.isPlaying,
      currentTime: this.currentTime,
    });
    this.updateMediaSessionState();
  }
  onPlaying() {
    this.isBuffering = false;
  }
  onBuffering() {
    this.logger.log('Buffering');
    this.isBuffering = true;
  }
  onPause() {
    this.isPlaying = false;
    this.pdPlayback.emit({
      playing: this.isPlaying,
      currentTime: this.currentTime,
    });
    this.updateMediaSessionState();
  }
  onPlaybackError(error) {
    this.logger.log('Playback error', error);
    this.playError = true;
    this.isPlaying = false;
    this.updateMediaSessionState();
  }
  getMediaMetadata() {
    const data = {
      title: this.episode.title,
      artist: this.episode.show.title,
      album: 'Poductivity',
    };
    const image = this.episode.imageUrl || this.episode.show.imageUrl;
    const mime = guessImageMimeFromPath(image);
    if (image && mime) {
      data.artwork = [
        {
          src: image,
          sizes: '500x500',
          type: mime,
        },
      ];
    }
    return new MediaMetadata(data);
  }
  updateMediaSessionState() {
    if (navigator && 'mediaSession' in navigator) {
      navigator.mediaSession.metadata = this.getMediaMetadata();
      navigator.mediaSession.playbackState = this.isPlaying
        ? 'playing'
        : 'paused';
      navigator.mediaSession.setPositionState({
        duration: this.duration,
        playbackRate: this.playbackRate,
        position: this.currentTime,
      });
    }
  }
  mediaSessionSeekBackward(details) {
    this.seekRelative(-details.seekOffset || -10);
  }
  mediaSessionSeekForward(details) {
    this.seekRelative(details.seekOffset || 10);
  }
  mediaSessionSeekTo(details) {
    this.seekTo(details.seekTime || 0, !!details.fastSeek);
  }
  registerMediaSession() {
    this.addMediaSessionHandler('play', this.play);
    this.addMediaSessionHandler('pause', this.pause);
    this.addMediaSessionHandler('seekbackward', this.mediaSessionSeekBackward);
    this.addMediaSessionHandler('seekforward', this.mediaSessionSeekForward);
    this.addMediaSessionHandler('seekto', this.mediaSessionSeekTo);
  }
  addMediaSessionHandler(action, handler) {
    try {
      navigator.mediaSession.setActionHandler(action, handler.bind(this));
    }
    catch (e) {
      this.logger.log(`The media session action, ${action}, is not supported`);
    }
  }
  cssVars() {
    return `
    :host {
      ${this.accentColor ? `--poductivity-accent: ${this.accentColor};` : ''}
    }
    `;
  }
  render() {
    var _a, _b, _c, _d, _e, _f, _g;
    const nodes = [];
    nodes.push(h("style", null, this.cssVars()));
    nodes.push(h("div", { class: this.wrapperClasses },
      h("div", { class: "art-wrapper", style: this.episode &&
          (this.episode.imageUrl || this.episode.show.imageUrl)
          ? {
            backgroundImage: `url(${this.episode.imageUrl || this.episode.show.imageUrl})`,
          }
          : {} },
        h("poductivity-client", { ref: (el) => {
            this.client = el;
          }, timestamp: this.currentTime, onLoadingEpisodeData: this.onDataLoading, onLoadedEpisodeData: this.onDataLoaded, onLoadingEpisodeDataFailed: this.onDataLoadingFailed, debug: this.debug })),
      h("div", { class: "main" },
        h("audio", { ref: (el) => {
            this.audio = el;
          }, preload: "none", controls: false, onPlay: this.onPlay, onPlaying: this.onPlaying, onPause: this.onPause, onWaiting: this.onBuffering, onTimeUpdate: this.onTimeUpdate, onError: this.onPlaybackError },
          h("source", { type: "audio/mpeg", src: (_a = this.episode) === null || _a === void 0 ? void 0 : _a.mediaUrl })),
        h("h1", { class: "title" }, ((_c = (_b = this.episode) === null || _b === void 0 ? void 0 : _b.show) === null || _c === void 0 ? void 0 : _c.websiteUrl) ? (h("a", { href: (_e = (_d = this.episode) === null || _d === void 0 ? void 0 : _d.show) === null || _e === void 0 ? void 0 : _e.websiteUrl, target: "_blank" }, this.title)) : (this.title)),
        h("h2", { class: "subtitle" },
          h("poductivity-marquee", { active: this.isPlaying }, this.subtitle)),
        h("div", { class: "subtext" },
          h("button", { class: "pd-button-link notes-button", onClick: this.showEpisodeNotes }, "Episode description"),
          this.enablePrivacyControl && (h("poductivity-opt-out-control", { ref: (el) => {
              this.optOutControl = el;
            }, up: true }))),
        h("poductivity-modal", { ref: (el) => {
            this.notesModal = el;
          }, absolute: true, fill: true, debug: this.debug },
          h("p", { class: "subtitle" }, (_f = this.episode) === null || _f === void 0 ? void 0 : _f.title),
          h("div", { innerHTML: ((_g = this.episode) === null || _g === void 0 ? void 0 : _g.description) || 'This episode has no notes.' })),
        h("div", { class: "controls" },
          h("button", { class: "btn-controls btn-seek btn-seek-back", title: "Seek back", disabled: !this.ready, onClick: this.seekBack, innerHTML: iconSeekBack }),
          h("button", { class: 'btn-controls btn-play ' + (this.isPlaying ? 'btn-pause' : ''), title: this.isPlaying ? 'Pause' : 'Play', disabled: !this.ready, onClick: this.togglePlay, innerHTML: this.isPlaying ? iconPause : iconPlay }),
          h("button", { class: "btn-controls btn-seek btn-seek-forward", title: "Seek forward", disabled: !this.ready, onClick: this.seekForward, innerHTML: iconSeekForward }),
          h("poductivity-playback-rate-control", { "playback-rate": this.playbackRate, onPlaybackRateChange: this.onPlaybackRateChanged }),
          h("div", { class: "controls-main" },
            h("poductivity-input-range", { min: 0, max: this.duration, step: 0.01, value: this.currentTime, disabled: !this.ready, onInputRangeChanged: this.onSeeked, onInputRangeInteracting: this.onSeeking }),
            h("div", { class: "time" },
              h("span", { class: "time-current" }, this.currentTimeFormatted),
              h("span", { class: "buffering-message", "aria-live": "off" }, "Buffering\u2026"),
              h("span", { class: "time-duration" }, this.durationFormatted)))),
        h("a", { href: "https://poductivity.com", target: "_blank", class: "pd-logo hide-text", innerHTML: logoSvg }))));
    // eslint-disable-next-line @stencil/render-returns-host
    return nodes;
  }
  static get is() { return "poductivity-player"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["./style/pd-player.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["./style/pd-player.css"]
  }; }
  static get assetsDirs() { return ["assets"]; }
  static get properties() { return {
    "episodeId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Poductivity Episode ID. Either this or feedUrl and episodeGuid are required"
      },
      "attribute": "episode-id",
      "reflect": false
    },
    "feedUrl": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Feed URL"
      },
      "attribute": "feed-url",
      "reflect": false
    },
    "episodeGuid": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Episode GUID as specified in the feed"
      },
      "attribute": "episode-guid",
      "reflect": false
    },
    "accentColor": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Accent colour. Supply as any CSS supported colour format, eg. #ff0000"
      },
      "attribute": "accent-color",
      "reflect": false
    },
    "horizontal": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Should the player be horizontal?"
      },
      "attribute": "horizontal",
      "reflect": false,
      "defaultValue": "false"
    },
    "theme": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'dark' | 'light' | 'poductivity'",
        "resolved": "\"dark\" | \"light\" | \"poductivity\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify the theme to use. Defaults to 'light'."
      },
      "attribute": "theme",
      "reflect": false,
      "defaultValue": "'light'"
    },
    "useOsFont": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Use the Operating System's default UI font instead of\nthe Poductivity brand font"
      },
      "attribute": "use-os-font",
      "reflect": false,
      "defaultValue": "false"
    },
    "debug": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable Debug mode. For internal use only"
      },
      "attribute": "debug",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "episode": {},
    "isPlaying": {},
    "isBuffering": {},
    "currentTime": {},
    "playbackRate": {},
    "isLoading": {},
    "loadingFailed": {},
    "playError": {}
  }; }
  static get events() { return [{
      "method": "pdPlayback",
      "name": "pdPlayback",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when playback is started or paused"
      },
      "complexType": {
        "original": "PlaybackEventData",
        "resolved": "PlaybackEventData",
        "references": {
          "PlaybackEventData": {
            "location": "import",
            "path": "../../events.interface"
          }
        }
      }
    }]; }
  static get methods() { return {
    "play": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Begin or resume playback",
        "tags": []
      }
    },
    "pause": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Pause playback",
        "tags": []
      }
    },
    "togglePlay": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Either play or pause playback, depending on the current playback state",
        "tags": []
      }
    },
    "seekTo": {
      "complexType": {
        "signature": "(timestamp: number | string, fastSeek?: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Seek to a specific timestamp in seconds",
        "tags": []
      }
    },
    "seekRelative": {
      "complexType": {
        "signature": "(seconds: number | string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Seek to a timestamp relative to the current playback position in seconds",
        "tags": []
      }
    },
    "setPlaybackRate": {
      "complexType": {
        "signature": "(playbackRate: number) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show the episode notes panel",
        "tags": []
      }
    },
    "showEpisodeNotes": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show the episode notes panel",
        "tags": []
      }
    },
    "hideEpisodeNotes": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Hide the episode notes panel",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "episodeGuid",
      "methodName": "setEpisodeParams"
    }, {
      "propName": "episodeId",
      "methodName": "setEpisodeParams"
    }, {
      "propName": "feedUrl",
      "methodName": "setEpisodeParams"
    }]; }
}
