import { Component, h, Host, Method, Prop, State } from '@stencil/core';
import { tryScheme } from '../../utils/httpUtils';
import { Logger } from '../../utils/Logger';
export class PdSubscribe {
  constructor() {
    /**
     * Required. Data for the show to subscribe to
     */
    this.showData = null;
    /**
     * Required. The browser/app's user agent. Used for feature detection
     */
    this.userAgent = '';
    /**
     * Position the modal absolute rather than fixed
     */
    this.absolute = false;
    /**
     * Fill all available space - no visible backdrop
     */
    this.fill = false;
    /**
     * Debug mode
     */
    this.debug = false;
    this.waiting = false;
    this.logger = new Logger('poductivity-subscribe', this.debug);
    this.linkClicked = this.linkClicked.bind(this);
  }
  /**
   * Attempt to open the default app
   */
  async attemptOpenDefaultApp() {
    try {
      this.waiting = true;
      if (!this.applePodcastsUrl) {
        throw 'Apple Podcasts ID not set';
      }
      if (this.isAndroid) {
        throw 'Android device';
      }
      await tryScheme(this.applePodcastsUrl);
      this.show();
      this.logger.log('Default app opened');
      this.waiting = false;
      return true;
    }
    catch (e) {
      this.waiting = false;
      this.logger.log('Failed to open default app', e);
      return this.show();
    }
  }
  /**
   * Show the modal
   */
  async show() {
    return this.modal.show();
  }
  linkClicked() {
    this.modal.hide(true);
  }
  get fullFeedUrl() {
    return `${this.showData.feedProtocol}://${this.showData.feedUrl}`;
  }
  get applePodcastsUrl() {
    const protocol = this.isAppleHandheld || this.isMac ? 'podcasts' : 'https';
    return this.showData.applePodcastsId
      ? `${protocol}://podcasts.apple.com/us/podcast/id${this.showData.applePodcastsId}?app=podcast`
      : '';
  }
  get spotifyUrl() {
    return this.showData.spotifyId
      ? `https://open.spotify.com/show/${this.showData.spotifyId}`
      : '';
  }
  get spotifyUri() {
    return this.showData.spotifyId
      ? `spotify://show:${this.showData.spotifyId}`
      : '';
  }
  get googlePodcastsUrl() {
    return `https://podcasts.google.com/feed/${btoa(this.fullFeedUrl)}`;
  }
  get pocketCastsUrl() {
    return this.showData.applePodcastsId
      ? `https://pca.st/itunes/${this.showData.applePodcastsId}`
      : '';
  }
  get overcastUrl() {
    return this.showData.applePodcastsId
      ? `https://overcast.fm/itunes${this.showData.applePodcastsId}`
      : '';
  }
  get isAppleHandheld() {
    return /(iPhone|iPad|iPod)/.test(this.userAgent);
  }
  get isMac() {
    return /Macintosh/i.test(this.userAgent);
  }
  get isAndroid() {
    return /android/i.test(this.userAgent);
  }
  getButton(url, content) {
    return (h("a", { href: url, target: "_blank", class: "pd-button pd-button-large pd-subscribe-button", onClick: this.linkClicked }, content));
  }
  render() {
    return (h(Host, null,
      this.waiting && (h("div", { class: "pd-subscribe-waiting" }, "Loading\u2026")),
      h("poductivity-modal", { ref: (el) => (this.modal = el), absolute: this.absolute, fill: this.fill, debug: this.debug },
        !this.isAndroid &&
          this.applePodcastsUrl &&
          this.getButton(this.applePodcastsUrl, 'Apple Podcasts'),
        this.getButton(this.fullFeedUrl, 'Feed URL'),
        this.getButton(this.googlePodcastsUrl, 'Google Podcasts'),
        this.isAppleHandheld &&
          this.overcastUrl &&
          this.getButton(this.overcastUrl, 'Overcast'),
        this.pocketCastsUrl &&
          this.getButton(this.pocketCastsUrl, 'Pocket Casts'),
        this.spotifyUrl && this.getButton(this.spotifyUrl, 'Spotify'))));
  }
  static get is() { return "poductivity-subscribe"; }
  static get originalStyleUrls() { return {
    "$": ["./pd-subscribe.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pd-subscribe.css"]
  }; }
  static get properties() { return {
    "showData": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "SubscribeShowData | null",
        "resolved": "SubscribeShowData",
        "references": {
          "SubscribeShowData": {
            "location": "import",
            "path": "../../episode.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Required. Data for the show to subscribe to"
      },
      "defaultValue": "null"
    },
    "userAgent": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Required. The browser/app's user agent. Used for feature detection"
      },
      "attribute": "user-agent",
      "reflect": false,
      "defaultValue": "''"
    },
    "absolute": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Position the modal absolute rather than fixed"
      },
      "attribute": "absolute",
      "reflect": false,
      "defaultValue": "false"
    },
    "fill": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Fill all available space - no visible backdrop"
      },
      "attribute": "fill",
      "reflect": false,
      "defaultValue": "false"
    },
    "debug": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Debug mode"
      },
      "attribute": "debug",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "waiting": {}
  }; }
  static get methods() { return {
    "attemptOpenDefaultApp": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Attempt to open the default app",
        "tags": []
      }
    },
    "show": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Show the modal",
        "tags": []
      }
    }
  }; }
}
