import { Logger } from './Logger';
import { sleep } from './utils';
import { LISTENERS_SERVER } from './config';
import { Build } from '@stencil/core';
import { checkMessageOrigin } from './httpUtils';
class DeviceService {
  constructor(debug) {
    this.debug = typeof debug === 'boolean' ? debug : Build.isDev;
    this.logger = new Logger('DeviceService', debug);
    this.logger.log('Created device class instance');
  }
  /**
   * listenerAccessToken will only be set if this is running
   * on a trusted domain - otherwise will be undefined
   */
  getDeviceInfo(maxTime = 10000) {
    return new Promise(async (resolve, reject) => {
      let exists = this.getCache();
      if (exists && !exists.working) {
        return resolve({
          deviceIdentifier: exists.deviceIdentifier,
          listenerAccessToken: exists.listenerAccessToken,
        });
      }
      if (exists && exists.working) {
        do {
          //TODO: can we do this without polling?
          await sleep(10);
          exists = this.getCache();
        } while (exists.working);
        if (exists.failed) {
          return reject('Already failed to get device identifier');
        }
        return resolve({
          deviceIdentifier: exists.deviceIdentifier,
          listenerAccessToken: exists.listenerAccessToken,
        });
      }
      this.logger.log('Initialising identifier frame');
      const idFrame = this.createIdentifierFrame();
      this.setCache({
        deviceIdentifier: '',
        working: true,
        failed: false,
      });
      const timeout = setTimeout(() => {
        this.setCache({
          deviceIdentifier: '',
          working: false,
          failed: true,
        });
        reject('Unable to get device identifier');
      }, maxTime);
      window.addEventListener('message', (e) => {
        var _a;
        if (checkMessageOrigin(e.origin) && ((_a = e.data) === null || _a === void 0 ? void 0 : _a.deviceIdentifier)) {
          clearTimeout(timeout);
          this.logger.log('Receiving local storage device info', e);
          resolve({
            deviceIdentifier: e.data.deviceIdentifier,
            listenerAccessToken: e.data.listenerAccessToken,
          });
          this.setCache({
            deviceIdentifier: e.data.deviceIdentifier,
            working: false,
            failed: false,
          });
          // document.body.removeChild(iframe);
        }
      });
      document.body.appendChild(idFrame);
    });
  }
  getCache() {
    return this.cache;
  }
  setCache(item) {
    this.cache = item;
  }
  getFrameSrc() {
    return `${LISTENERS_SERVER}/v1/devices/id-frame${this.debug ? '?debug' : ''}`;
  }
  createIdentifierFrame() {
    const iframe = document.createElement('iframe');
    iframe.src = this.getFrameSrc();
    iframe.tabIndex = -1;
    iframe.style.display = 'block';
    iframe.style.visibility = 'hidden';
    iframe.style.width = '0';
    iframe.style.height = '0';
    iframe.style.border = '0';
    iframe.setAttribute('aria-hidden', 'true');
    return iframe;
  }
}
export default new DeviceService();
