import { EventEmitter } from '../../stencil-public-runtime';
import { OptionData, OptionGroupData, OptOutType, PdData } from '../../slot.interface';
import { SubscribeShowData } from '../../episode.interface';
import { AudioControlEventData, FeedSubscribeData, LoadedEpisodeData, LoadingEpisodeDataFailed, OpenExternalUrlData, OptOutEventData, PerformActionData } from '../../events.interface';
export declare class PdClient {
  /**
   * Pass a reference to a <audio> element to have the
   * client attach the necessary listeners and perform
   * control action for you
   */
  audioElement: HTMLAudioElement;
  /**
   * If not using the audioElement prop, pass the current audio timestamp here
   */
  timestamp: number;
  /**
   * If not using the audioElement prop, pass the current playback state here
   */
  isPlaying: boolean;
  /**
   * If not using the audioElement prop, pass the current playback rate here
   */
  playbackRate: number;
  /**
   * If integrating into an app, pass the device's native advertising ID here.
   * If integrating into a web app, the client will handle generation
   * and persistence of a device identifier for you
   */
  deviceIdentifier: string;
  /**
   * The User Agent your app uses.
   * Only necessary when integrating into an app with a custom user agent.
   * Will use the current browser's user agent if available.
   */
  userAgent: string;
  /**
   * Optionally supply a <poductivity-opt-out-control> element positioned
   * anywhere in the DOM. Will show inside this element otherwise
   */
  optOutControl: HTMLPoductivityOptOutControlElement | null;
  /**
   * Use to disable the standard Poductivity subscribe behaviour if you wish
   * to capture and handle the subscribeToFeed event yourself
   */
  noSubscribe: boolean;
  /**
   * Optionally supply a <poductivity-messages> element positioned
   * anywhere in the DOM. Will show in this element otherwise
   */
  messagesElement: HTMLPoductivityMessagesElement | null;
  /**
   * Enable debug mode. Shows a status panel in the client and adds
   * extra console logging. For internal use only
   */
  debug: boolean;
  /**
   * Pass in a data object to use instead of contacting the API server
   * For internal/debug use only
   */
  data: PdData | null;
  /**
   * Emitted when the episode data is about to load
   */
  loadingEpisodeData: EventEmitter<void>;
  /**
   * Emitted when the episode data is loaded
   */
  loadedEpisodeData: EventEmitter<LoadedEpisodeData>;
  /**
   * Emitted if the episode data fails to load
   */
  loadingEpisodeDataFailed: EventEmitter<LoadingEpisodeDataFailed>;
  /**
   * Emitted when an action is performed
   * Only required if you want your app to also perform an action as a result
   */
  performActionEvent: EventEmitter<PerformActionData>;
  /**
   * Emitted when an action requests a change to audio control.
   * Only required if not using the audioElement prop
   */
  audioControlEvent: EventEmitter<AudioControlEventData>;
  /**
   * Emitted when an action requests to open a URL in a new window.
   * Only required when integrating into an app that won't support window.open
   */
  openExternalUrl: EventEmitter<OpenExternalUrlData>;
  /**
   * Emitted when an action requests to subscribe to a feed.
   * Only required when wishing to override the default modal
   */
  subscribeToFeed: EventEmitter<FeedSubscribeData>;
  private episode;
  private slots;
  optionGroups: OptionGroupData[];
  private optOut;
  protected loading: boolean;
  private loaded;
  private timestampI;
  protected isPlayingI: boolean;
  private internalDeviceIdentifier;
  dynamicOptionGroup: OptionGroupData | null;
  modalElement: HTMLPoductivityModalElement;
  subscribeElement?: HTMLPoductivitySubscribeElement;
  private clientService;
  private optOutService;
  private logger;
  private lastClose;
  private contiguousPlaybackStart;
  private playbackStarted;
  private lastTimestampUpdateAt;
  private enablePrivacyControl;
  private actionHandler;
  private privacySubscription;
  private version;
  constructor();
  componentWillLoad(): void;
  componentDidRender(): void;
  disconnectedCallback(): void;
  /**
   * Set the current episode by feed URL and episode GUID.
   */
  setEpisodeByFeed(feedUrl: string, episodeGuid: string): Promise<void>;
  /**
   * Set the current episode by Poductivity Episode ID
   */
  setEpisodeById(episodeId: string): Promise<void>;
  /**
   * If integrating into an app, call this before destroying
   * the webview to allow the client to flush it's caches.
   * If integrating into the web, the client handles this itself
   * using the Browser Visibility API
   */
  close(): Promise<void>;
  /**
   * Is the client ready?
   */
  isReady(): Promise<boolean>;
  protected get hasLoadedData(): boolean;
  protected get subscribeData(): SubscribeShowData;
  protected get playbackRateI(): number;
  protected get wrapperClasses(): string;
  protected staticDataChanged(data: PdData): Promise<void>;
  private loadAll;
  private loadData;
  private subscribeToOptOuts;
  protected optOutChanged(val: OptOutType): void;
  protected onPageHide(): void;
  protected onVisibilityChange(): void;
  protected changeOptOut(event: CustomEvent<OptOutEventData>): Promise<void>;
  protected debugChanged(debug: boolean): void;
  protected userAgentChanged(userAgent: string): void;
  protected setDeviceIdentifier(suppliedDeviceIdentifier: string): Promise<void>;
  protected updateAudioElementListeners(newEl: HTMLAudioElement, oldEl?: HTMLAudioElement): void;
  protected updateExternalOptOutControlListeners(newEl: HTMLPoductivityOptOutControlElement | null, oldEl?: HTMLPoductivityOptOutControlElement | null): void;
  private audioElPlaying;
  private audioElPaused;
  isPlayingPropChanged(): void;
  internalIsPlayingChanged(isPlaying: boolean): void;
  private audioElTimeUpdate;
  timestampPropChanged(): void;
  internalTimestampChanged(newTimestamp: number, oldTimestamp: number): void;
  messagesElementChanged(messagesElement: HTMLPoductivityMessagesElement | null): void;
  private currentSlots;
  private currentOptionGroups;
  private currentActionableOptionGroups;
  editOptionGroup(optionGroupId: string, changes: any): OptionGroupData | null;
  editOption(optionGroupId: string, optionId: string, changes: any): OptionData | null;
  private recordPlaybackPortion;
  private recordAction;
  private perform;
  controlAudio(action: 'pause' | 'resume'): void;
  controlAudio(action: 'seekAbsolute' | 'seekRelative', val: number): void;
  private getDebugPanel;
  private getCurrentOptionGroupsUI;
  private getOptionGroupButtons;
  render(): any;
}
